/*----------------------------------------------------------------------------*/
/* Copyright (c) 2017-2018 FIRST. All Rights Reserved.                        */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

package team.frc.morpheus.robot;

import edu.wpi.cscore.VideoMode;
import edu.wpi.first.cameraserver.CameraServer;
import edu.wpi.first.wpilibj.TimedRobot;
import edu.wpi.first.wpilibj.command.Command;
import edu.wpi.first.wpilibj.command.CommandGroup;
import edu.wpi.first.wpilibj.command.Scheduler;
import edu.wpi.first.wpilibj.command.WaitCommand;
import edu.wpi.first.wpilibj.smartdashboard.SendableChooser;
import edu.wpi.first.wpilibj.smartdashboard.SmartDashboard;
import team.frc.morpheus.robot.commands.drivebase.DriverControl;
import team.frc.morpheus.robot.commands.drivebase.*;
import team.frc.morpheus.robot.commands.drivebase.features.AssistedEnhanced;
import team.frc.morpheus.robot.subsystems.DriveBase;
import team.frc.morpheus.vision.pixy2.USBPixy;

import java.io.File;


/**
 * The VM is configured to automatically run this class, and to call the
 * functions corresponding to each mode, as described in the TimedRobot
 * documentation. If you change the name of this class or the package after
 * creating this project, you must also update the build.properties file in the
 * project.
 */
public class Robot extends TimedRobot {

    public enum Type {
        PRACTICE,
        COMPETITION
    }

    public static final DriveBase driveBase = DriveBase.getInstance();
    public static OI m_oi;

    private boolean autoTeleOpStarted;

    Command m_autonomousCommand;
    SendableChooser<Command> m_chooser = new SendableChooser<>();

    /**
     * This function is run when the robot is first started up and should be
     * used for any initialization code.
     */
    @Override
    public void robotInit() {
        m_oi = new OI();
        CameraServer.getInstance().startAutomaticCapture(0).setVideoMode(new VideoMode(VideoMode.PixelFormat.kYUYV, 320, 180, 30));
        //USBPixy.init();

        try {
            m_chooser.addOption("Test Trajectory", new ExecuteCSVTrajectory(new File("/home/lvuser/1meterstraight.csv")));
        } catch (Exception e) {
            e.printStackTrace();
        }

        /* Looks deprecated but is still required for some reason! */
        m_chooser.addDefault("Teleop Auto", new CommandGroup()); // no-op auto; put your auto here to run before teleop
        SmartDashboard.putData("Autonomous Chooser", m_chooser);
    }

    /**
     * This function is called once each time the robot enters Disabled mode.
     * You can use it to reset any subsystem information you want to clear when
     * the robot is disabled.
     */
    @Override
    public void disabledInit() {

    }

    @Override
    public void disabledPeriodic() {
        Scheduler.getInstance().run();
    }

    /**
     * This autonomous (along with the chooser code above) shows how to select
     * between different autonomous modes using the dashboard. The sendable
     * chooser code works with the Java SmartDashboard. If you prefer the
     * LabVIEW Dashboard, remove all of the chooser code and uncomment the
     * getString code to get the auto name from the text box below the Gyro
     *
     * <p>You can add additional auto modes by adding additional commands to the
     * chooser code above (like the commented example) or additional comparisons
 * to the switch structure below with additional strings & commands.
     */
    @Override
    public void autonomousInit() {
        CommandGroup autonomousCommand = new CommandGroup();
        autonomousCommand.addSequential(m_chooser.getSelected());
        autonomousCommand.addSequential(new WaitCommand(.1));
        autonomousCommand.addSequential(new DisableBrakes());
        m_autonomousCommand = autonomousCommand;

        driveBase.enableBrakes();

        // schedule the autonomous command
        if (m_autonomousCommand != null) {
            m_autonomousCommand.start();
        }
    }

    /**
     * This function is called periodically during autonomous.
     */
    @Override
    public void autonomousPeriodic() {
        Scheduler.getInstance().run();
        if (m_autonomousCommand.isCompleted()) {
            m_autonomousCommand = new DriverControl().withFeatures(DriverControl.Features.SLOWMODE, new AssistedEnhanced());
            m_autonomousCommand.start();
        }
    }

    @Override
    public void teleopInit() {
        // This makes sure that the autonomous stops running when
        // teleop starts running. If you want the autonomous to
        // continue until interrupted by another command, remove
        // this line or comment it out.
        if (m_autonomousCommand != null) {
            m_autonomousCommand.cancel();
        }
        driveBase.disableBrakes();
        //new DriverControl().withFeature(DriverControl.Features.ASSISTED).start();
        new DriverControl().withFeatures(DriverControl.Features.SLOWMODE, new AssistedEnhanced()).start();
    }

    /**
     * This function is called periodically during operator control.
     */
    @Override
    public void teleopPeriodic() {
        Scheduler.getInstance().run();
    }

    /**
     * This function is called periodically during test mode.
     */
    @Override
    public void testPeriodic() {
    }
}
