/*----------------------------------------------------------------------------*/
/* Copyright (c) 2017-2018 FIRST. All Rights Reserved.                        */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

package team.frc.morpheus.robot;

import team.frc.morpheus.robot.OI.Driver;
import team.frc.morpheus.robot.commands.drivebase.DriverControl;

/**
 * The RobotMap is a mapping from the ports sensors and actuators are wired into
 * to a variable name. This provides flexibility changing wiring, makes checking
 * the wiring easier and significantly reduces the number of magic numbers
 * floating around.
 */
public class RobotMap {

    public static final Robot.Type TYPE = Robot.Type.COMPETITION;

    /* Drivebase */
    public static final int LEFT_DRIVE = 1; //CAN // 2
    public static final int LEFT_FOLLOWER = 3; //CAN // 4
    public static final int RIGHT_DRIVE = 2; //CAN // 1
    public static final int RIGHT_FOLLOWER = 4; //CAN // 3
    public static final int SHIFT = (TYPE == Robot.Type.COMPETITION) ? 3 : 4; //PCM
    public static final int LEFT_CHANNEL_A = (TYPE == Robot.Type.COMPETITION) ? 7 : 9;
    public static final int LEFT_CHANNEL_B = (TYPE == Robot.Type.COMPETITION) ? 6 : 8;
    public static final int RIGHT_CHANNEL_A = (TYPE == Robot.Type.COMPETITION) ? 9 : 7;
    public static final int RIGHT_CHANNEL_B = (TYPE == Robot.Type.COMPETITION) ? 8 : 6;



    /* Other */
    public static final int PCM = 61;
    public static final int PDP = 62;

    /* Flags */
    public static final Driver DRIVER = Driver.GARRETT;

    public static final boolean INTAKE_SIDE_FORWARD = false;
    public static final double ASSISTED_DRIVING_KP = .9;
    public static final double ASSISTED_DRIVING_KI = 0.0;
    public static final double ASSISTED_DRIVING_KD = 3.0;
    public static final DriverControl.Feature[] DRIVER_CONTROL_ENABLED_FEATURES = {
            DriverControl.Features.ASSISTED
    };

}
