package team.frc.morpheus.robot.commands.drivebase;

import edu.wpi.first.wpilibj.Timer;
import edu.wpi.first.wpilibj.command.Command;
import team.frc.morpheus.robot.Robot;
import team.frc.morpheus.util.input.Coordinate;

import java.util.Optional;

/**
 * Takes inches not feet
 */

public class DriveStraight extends Command {
    double targetPosition, currentPosition;
    boolean reverse = false;
    double inches;
    Timer timer;

    Optional<Double> timeout = Optional.empty();

    public DriveStraight(double inches) {
        requires(Robot.driveBase);
        if (inches < 0) reverse = true;
        this.inches = inches;
    }

    public DriveStraight(double inches, double timeout) {
        requires(Robot.driveBase);
        if (inches < 0) reverse = true;
        this.inches = inches;
        this.timeout = Optional.of(timeout);
        timer = new Timer();
    }

    // Called just before this Command runs the first time
    protected void initialize() {
        targetPosition = inches + Robot.driveBase.getLeftPosition();
        Robot.driveBase.set(new Coordinate.Cartesian(0, reverse ? -0.42 : 0.42));
        if (timeout.isPresent()) {
            timer.start();
        }
    }

    // Called repeatedly when this Command is scheduled to run
    protected void execute() {
        currentPosition = Robot.driveBase.getLeftPosition();
        System.out.println("This is left: " + Robot.driveBase.getLeftPosition());
        System.out.println("This is right: " + Robot.driveBase.getRightPosition());
    }

    // Make this return true when this Command no longer needs to run execute()
    protected boolean isFinished() {
        if (timeout.isPresent()) {
            if (reverse) {
                return currentPosition <= targetPosition || timer.get() > timeout.get();
            } else {
                return currentPosition >= targetPosition || timer.get() > timeout.get();
            }
        } else {
            if (reverse) {
                return currentPosition <= targetPosition;
            } else {
                return currentPosition >= targetPosition;
            }
        }
    }

    // Called once after isFinished returns true
    protected void end() {
        Robot.driveBase.set(new Coordinate.Cartesian(0, 0));
        if (timeout.isPresent()) {
            timer.stop();
        }
    }
}