package team.frc.morpheus.robot.commands.drivebase;

import edu.wpi.first.wpilibj.command.Command;
import team.frc.morpheus.robot.Robot;
import team.frc.morpheus.robot.RobotMap;
import team.frc.morpheus.util.input.Coordinate;
import team.frc.morpheus.util.input.Filters;
import team.frc.morpheus.vision.limelight.DoubleTargetTrackerLimelight;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.function.Function;

/**
 *
 */
public class DriverControl extends Command {

    @FunctionalInterface
    public interface Feature extends Function<Coordinate, Coordinate> {}

    private LinkedList<Feature> features;

    public DriverControl() {
        // Use requires() here to declare subsystem dependencies
        // eg. requires(chassis);
        requires(Robot.driveBase);
        features = new LinkedList<>();
    }

    // returns itself, so you can chain away!
    public DriverControl withFeature(Feature feat) {
        features.add(feat);
        return this;
    }

    public DriverControl withFeatures(Feature... feat) {
        features.addAll(Arrays.asList(feat));
        return this;
    }

    // Called just before this Command runs the first time
    protected void initialize() {
    }

    // Called repeatedly when this Command is scheduled to run
    protected void execute() {
        Coordinate input = Robot.m_oi.driver.getLeftStick();
        for (Feature feature : features) input = feature.apply(input);
        switch (RobotMap.DRIVER) {
            case DALER:
            default:
            case ISAAC:
                Robot.driveBase.set(Filters.threshold(input, 0.1));
                break;
        }


        //Robot.driveBase.setCheesy(Filters.threshold(Robot.m_oi.driver.getLeftStick(), 0.1), Robot.m_oi.driver.getButtonStart());
    }

    // Make this return true when this Command no longer needs to run execute()
    protected boolean isFinished() {
        // never finish
        return false;
    }

    // Called once after isFinished returns true
    protected void end() {
    }

    // Called when another command which requires one or more of the same
    // subsystems is scheduled to run
    protected void interrupted() {
        Robot.driveBase.set(new Coordinate.Cartesian(0, 0));
    }

    public static class Features {

        public static Feature ASSISTED = (Coordinate in) -> {
            double correctionExtent = Robot.m_oi.driver.getRightTrigger();
            return new Coordinate.Cartesian(in.x() + correctionExtent * (DoubleTargetTrackerLimelight.getLRAdjustment()) * RobotMap.ASSISTED_DRIVING_KP, in.y());
        };

        public static Feature SLOWMODE = (Coordinate in) -> {
            if (Robot.m_oi.driver.getLeftTrigger() > .4){
                return in;
            } else {
                switch(RobotMap.DRIVER) {
                    default:
                    case GARRETT:
                        return new Coordinate.Cartesian(in.x() / 2, in.y() * 0.7);
                    case ISAAC:
                        return new Coordinate.Cartesian(in.x() / 2, in.y());
                }
            }

        };

    }
}
