package team.frc.morpheus.util;

import edu.wpi.first.wpilibj.PIDController;
import edu.wpi.first.wpilibj.PIDOutput;
import edu.wpi.first.wpilibj.PIDSource;

public class PIDCController extends PIDController {

    private double activationThreshold, c;

    private class OutputShim implements PIDOutput {

        private PIDOutput actualOutput;

        public OutputShim(PIDOutput output) {
            this.actualOutput = output;
        }

        @Override
        public void pidWrite(double output) {
            if (Math.abs(PIDCController.this.getError()) < PIDCController.this.activationThreshold && !PIDCController.this.onTarget()) {
                actualOutput.pidWrite(output + Math.copySign(c, output));
            } else {
                actualOutput.pidWrite(output);
            }
        }

    }


    public PIDCController(double Kp, double Ki, double Kd, PIDSource source, PIDOutput output, double period) {
        super(Kp, Ki, Kd, source, output, period);
        super.m_pidOutput = new OutputShim(output);
    }

    public PIDCController(double Kp, double Ki, double Kd, PIDSource source, PIDOutput output) {
        super(Kp, Ki, Kd, source, output);
        super.m_pidOutput = new OutputShim(output);
    }

    public PIDCController(double Kp, double Ki, double Kd, double Kf, PIDSource source, PIDOutput output,
                          double period) {
        super(Kp, Ki, Kd, Kf, source, output, period);
        super.m_pidOutput = new OutputShim(output);
    }

    public PIDCController(double Kp, double Ki, double Kd, double Kf, PIDSource source, PIDOutput output) {
        super(Kp, Ki, Kd, Kf, source, output);
        super.m_pidOutput = new OutputShim(output);
    }

    public void configureC(double activationThreshold, double c) {
        this.activationThreshold = activationThreshold;
        this.c = c;
    }


}
