package team.frc.morpheus.util.transform;

public class TransformMatrix {

  private final double[] mat; // this has to be 16 long
  
  private TransformMatrix(double ...inmat) {
    mat = inmat;
  }

  public Point3D apply(Point3D vec) {
    return new Point3D(vec.x * mat[0] + vec.y * mat[1] + vec.z * mat[2] + vec.h * mat[3],
                       vec.x * mat[4] + vec.y * mat[5] + vec.z * mat[6] + vec.h * mat[7],
                       vec.x * mat[8] + vec.y * mat[9] + vec.z * mat[10] + vec.h * mat[11],
                       vec.x * mat[12] + vec.y * mat[13] + vec.z * mat[14] + vec.h * mat[15]);
  }

  private static double cos(double degrees) {
    return Math.cos(degrees*(Math.PI/180));
  }
  
  private static double sin(double degrees) {
    return Math.sin(degrees*(Math.PI/180));
  }

  public static TransformMatrix rotateX(double angleDegrees) {
    double mat[] = { 1, 0, 0, 0,
                     0, cos(angleDegrees), -sin(angleDegrees), 0,
                     0, sin(angleDegrees), cos(angleDegrees), 0,
                     0, 0, 0, 1};
    return new TransformMatrix(mat);
  }

  public static TransformMatrix rotateY(double angleDegrees) {
    double mat[] = { cos(angleDegrees), 0, sin(angleDegrees), 0,
                     0, 1, 0, 0,
                     -sin(angleDegrees), 0, cos(angleDegrees), 0,
                     0, 0, 0, 1};
    return new TransformMatrix(mat);
  }

  public static TransformMatrix rotateZ(double angleDegrees) {
    double mat[] = {cos(angleDegrees), -sin(angleDegrees), 0, 0,
                    sin(angleDegrees), cos(angleDegrees), 0, 0,
                    0, 0, 1, 0,
                    0, 0, 0, 1};
    return new TransformMatrix(mat);
  }

  @Override
  public String toString() {
    return String.format("[%g\t%g\t%g\t%g|\n"+
                         "|%g\t%g\t%g\t%g|\n"+
                         "|%g\t%g\t%g\t%g|\n"+
                         "|%g\t%g\t%g\t%g]\n",
                         mat[0], mat[1], mat[2], mat[3],
                         mat[4], mat[5], mat[6], mat[7],
                         mat[8], mat[9], mat[10], mat[11],
                         mat[12], mat[13], mat[14], mat[15]); // :/
  }

}