package team.frc.morpheus.vision.pixy2;

import edu.wpi.first.wpilibj.Filesystem;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

public class USBPixy {

    public final static int RES_X = 316;
    public final static int RES_Y = 208;

    private final static int DATA_SIZE = 8;

    static {
        try {
            System.load(Filesystem.getDeployDirectory().getAbsolutePath() + "/libusb.so");
            System.load(Filesystem.getDeployDirectory().getAbsolutePath() + "/libUSBPixy.so");
        } catch (UnsatisfiedLinkError e) {
            System.err.println(e.getMessage());
        }
        init();
    }

    public static class Block {

        public final int signature, x, y, width, height, angle, index, age;

        Block(int frame[], int offset) {
            signature = frame[offset];
            x = frame[offset + 1];
            y = frame[offset + 2];
            width = frame[offset + 3];
            height = frame[offset + 4];
            angle = frame[offset + 5];
            index = frame[offset + 6];
            age = frame[offset + 7];
        }

        Block(IntBuffer in) {
            signature = in.get();
            x = in.get();
            y = in.get();
            width = in.get();
            height = in.get();
            angle = in.get();
            index = in.get();
            age = in.get();
        }

        @Override
        public String toString() {
            return String.format("%d: (%d, %d, %d, %d)", signature, x, y, width, height);
        }

    }

    public static int init() {
        return glue_init();
    }

    public static void close() {
        glue_close();
    }

    public static boolean isAvailable() {
        return glue_isAvailable();
    }

    /* TODO: optimize with "flippable" bytebuffer to avoid wasting memory */
    public static Block[] getBlocks() {
        ByteBuffer buf = glue_getBlocks(); // TODO: handle null better
        buf.order(ByteOrder.LITTLE_ENDIAN);
        IntBuffer in = buf.asIntBuffer();
        Block blocks[] = new Block[in.capacity() / 8];
        for (int i = 0; i < blocks.length; i++)
            blocks[i] = new Block(in);
        return blocks;
    }

    public static Block getLargestBlock() {
        int temp[] = glue_getLargestBlock();
        return new Block(temp, 0);
    }

    private static native int glue_init();

    private static native void glue_close();

    private static native boolean glue_isAvailable();

    private static native int[] glue_getLargestBlock();

    private static native ByteBuffer glue_getBlocks();
}
